from itertools import count
from __aplicarUps import aplicarUps
from __expandir import expandir
from __hashA import hashA
from __hashB import hashB
from __mixradix import mixradix


def crearfuncion_firmar (M, basesAB, basesCD, zetas, zetasAB, zetasCD, comprimido, mbits, upbits):

	largo = len (zetas)

	def firmar (mensaje, priv):
		mensajeBytes = mensaje if type (mensaje) == bytes else mensaje.encode ('utf-8')
		clave = expandir (priv['clave'], upbits, largo) if comprimido else priv['clave']
		sal = priv['sal']

		# aplicar hash con sal al mensaje
		h = hashA (sal + mensajeBytes, mbits)

		# usar un contador hasta obtener una palabra valida
		for n in count ():
			nHex = b'%x' % n
			m = hashB (nHex + h, mbits)
			if m < M: break

		# calcular mensaje
		upsAB = mixradix (m, basesAB)
		downsAB = [zn - u for u, zn in zip (upsAB, zetasAB)]

		# calcular checksum
		check = sum (downsAB)
		upsCD = mixradix (check, basesCD)
		downsCD = [zn - u for u, zn in zip (upsCD, zetasCD)]

		# ups firmar
		ups = downsAB + downsCD

		# aplicar ups
		firma = aplicarUps (clave, ups, sal=sal, inicio=True, zetas=zetas)

		return firma, sal

	return firmar


# ----------------------------------------------------------------
if __name__ == '__main__':

	M, mbits = 10000000000000000, 54
	basesAB = 10, 10, 10, 10, 10, 10, 10, 10
	basesCD = 10, 10, 10, 10, 10, 10, 10, 10
	zetas = 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9
	zetasAB = 9, 9, 9, 9, 9, 9, 9, 9
	zetasCD = 9, 9, 9, 9, 9, 9, 9, 9
	upbits = 64

	firmar = crearfuncion_firmar (M, basesAB, basesCD, zetas, zetasAB, zetasCD, comprimido=True, mbits=mbits, upbits=upbits)

	priv = {'clave': b'abc', 'sal': b'123'}
	mensaje = 'abc'
	firma, sal = firmar (mensaje, priv)

	print ('firma = ' + repr (firma))

